-----------------------------------------------------------------------------------------------
-- Client Lua Script for SlingDing
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- SlingDing Module Definition
-----------------------------------------------------------------------------------------------
local SlingDing = {} 

local defaultSettings = {
	version = "1.3.1",
	bOverrideSaved = true,
	bLocked = false,
	bFocus = true,
	nResourceBarHeightPercent = 0.25,
	clrResourceBar = "ffa2cffe",
	clrResourceBarBG = "ff363737",
	clrFocusBar = "ffd75fdf",
	clrFocusBarBG = "ffcb0162",
	clrDivider = "804bec45",
	clrSurgeBG = "cce50000",
	clrFocusText = "ffffffff",
	clrResourceText = "ffffffff",
	sResourceBarTexture = "Gloss",
	sFocusBarTexture = "Minimalist",
	sDividerTexture = "Gloss",
	nOpacity = 1,
	nFlickerOpacity = 0.15,
	bShowSurge = true,
	bFocusText = true,
	bFocusTextLeft = false,
	bFocusTextCenter = true,
	bFocusTextRight = false,
	bFocusTextTop = false,
	bFocusTextVCenter = false,
	bFocusTextBottom = true,
	bResourceText = false,
	bResourceTextLeft = false,
	bResourceTextCenter = true,
	bResourceTextRight = false,
	bResourceTextTop = true,
	bResourceTextVCenter = false,
	bResourceTextBottom = false,
	bSmoothUpdate = false,
	bEditMode = false
}

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
local ktClassDividers = {
	[GameLib.CodeEnumClass.Warrior] 		= 0,
	[GameLib.CodeEnumClass.Engineer] 		= 2,
	[GameLib.CodeEnumClass.Esper] 			= 5,
	[GameLib.CodeEnumClass.Medic] 			= 4,
	[GameLib.CodeEnumClass.Stalker] 		= 0,
	[GameLib.CodeEnumClass.Spellslinger] 	= 4,
}

local ktClassToDividers = {
	[GameLib.CodeEnumClass.Warrior] 		= {},
	[GameLib.CodeEnumClass.Engineer] 		= { [1]= 0.3, 	[2]=0.7, },
	[GameLib.CodeEnumClass.Esper] 			= { [1]= 0.2, 	[2]=0.4,	[3]= 0.6,	[4]=0.8,	[5]= 1, },
	[GameLib.CodeEnumClass.Medic] 			= { [1]= 0.25,	[2]=0.5,	[3]= 0.75,	[4]=1, },
	[GameLib.CodeEnumClass.Stalker] 		= {},
	[GameLib.CodeEnumClass.Spellslinger] 	= { [1]= 0.25, 	[2]=0.5,	[3]= 0.75,	[4]=1, },
}
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SlingDing:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
    return o
end

function SlingDing:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "SlingDing"
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction ,strConfigureButtonText )
end
 

-----------------------------------------------------------------------------------------------
-- SlingDing OnLoad
-----------------------------------------------------------------------------------------------
function SlingDing:OnLoad()

	self.bCharacterCreated = false
	self.bRestored = false
	self.bWindowManagement = false
	self.nTryCount = 0
	
	Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
	
	Apollo.RegisterTimerHandler("RestoreBS", "OnCharacterCreated", self)
	if GameLib.GetPlayerUnit() ~= nil then
		self:OnCharacterCreated()
	else
		Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)
	end
end


function SlingDing:OnConfigure()
	if self.bIsLoaded then
		if self.SettingsOld == nil then
			self.SettingsOld = self:CopyTable(self.Settings)
		end

		self:SetOptions()
		self.wndConfig:Show(true)	
		self.Settings.bEditMode = true
	else
		Print("SlingDing is not available for this class.")
	end
end


-----------------------------------------------------------------------------------------------
-- SlingDing OnDocLoaded


function SlingDing:OnCharacterCreated()

	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer == nil or unitPlayer:GetHealth() == nil or self.bCharacterCreated then
		return
	end
	
	if not self.bRestored and  self.nTryCount < 5 then

			Apollo.CreateTimer("RestoreBS", 5, false)
			Apollo.StartTimer("RestoreBS")
			Print("SlingDing: Trying to load saved data!")--..self.nTryCount)
			self.nTryCount = self.nTryCount + 5
		return
	elseif not self.bRestored and  self.nTryCount >= 5 then
		if self.Settings == nil then
			--self.Settings = self:CopyTable(defaultSettings)
			Print("SlingDing: No settings found. Loading default settings.")
			self:ResetSettings()
			
			
		end
	
	end
	
	self.bCharacterCreated = true
	
	
	

	
	local bLoad = true
	
	self.playerClass = unitPlayer:GetClassId()
	if self.playerClass == GameLib.CodeEnumClass.Spellslinger then
		self.Resource = 4	
	elseif self.playerClass == GameLib.CodeEnumClass.Medic then
		self.Resource = 1
	elseif self.playerClass == GameLib.CodeEnumClass.Stalker then
		self.Resource = 3
	elseif self.playerClass == GameLib.CodeEnumClass.Esper then
		self.Resource = 1	
	elseif self.playerClass == GameLib.CodeEnumClass.Engineer then
		self.Resource = 1
	elseif self.playerClass == GameLib.CodeEnumClass.Warrior then
		self.Resource = 1	
	end
	
	if bLoad then
		if self.Settings == nil then
		--	self:ResetSettings()
		end
		Apollo.LoadSprites("SDBorders.xml", "SDBorders")
		self.xmlDoc = XmlDoc.CreateFromFile("SlingDing.xml")
		self.xmlDoc:RegisterCallback("OnDocLoaded", self)
		
		self.wndContainer = Apollo.LoadForm(self.xmlDoc, "SlingDingContainer" , "FixedHudStratum", self)
		if self.wndContainer == nil then
			Apollo.AddAddonErrorText(self, "Could not load the Container for some reason.")
			return
		end
		self.wndContainer:Show(true)
		
		self.wndFocusBar = self.wndContainer:FindChild("FocusBar")
		self.wndResourceBar = self.wndContainer:FindChild("ResourceBar")
		self.wndSurge = self.wndContainer:FindChild("Surged")
		self.wndFocusText = self.wndContainer:FindChild("FocusText")
		self.wndResourceText = self.wndContainer:FindChild("ResourceText")
		
		self.bIsMelee = false
		if self.playerClass == GameLib.CodeEnumClass.Stalker or self.playerClass == GameLib.CodeEnumClass.Engineer or self.playerClass == GameLib.CodeEnumClass.Warrior then
			self.Settings.bFocus = false
			self.bIsMelee = true

			
			if self.playerClass == GameLib.CodeEnumClass.Engineer then
				self.wndPetBar = Apollo.LoadForm(self.xmlDoc, "PetForm", self.wndContainer, self)
				self.wndPetBar:Show(true)
				
				self.wndPetBar:FindChild("StanceMenuOpenerBtn"):AttachWindow(self.wndPetBar:FindChild("StanceMenuBG"))
			
				for idx = 1, 5 do
					self.wndPetBar:FindChild("Stance"..idx):SetData(idx)
				end
				self:OnShowActionBarShortcut(1, IsActionBarSetVisible(1)) 
			end

		end
		
		
		self.tDividers = {}
		local nMax = #ktClassToDividers[self.playerClass] 
		if nMax ~= 0 then
			local nLeft = 0
			for k, nRight in pairs(ktClassToDividers[self.playerClass]) do
				self.tDividers[k] = Apollo.LoadForm(self.xmlDoc, "DividerTemp", self.wndResourceBar, self)
				self.tDividers[k]:SetAnchorPoints(nLeft, 0, nRight, 1)
				self.tDividers[k]:SetAnchorOffsets(0,0,0,0)
				
				
				if k ~= nMax or nRight <1 then
					local myPixie = {
						bLine = true,
						fWidth = 2.0,
						bRestart = true,
						cr = {a=1,r=0,g=0,b=0},
						loc = {
							fPoints = {nRight, 0, nRight, 1},
							nOffsets = {0,0,0,0}
						},
					}					
					self.wndResourceBar:AddPixie(myPixie)
				end
				nLeft = nRight
			end
		end
		--[[local nMax = ktClassDividers[self.playerClass]
				
		self.tDividers = {}
		
		if nMax > 2 then
			local nLeft, nRight
			for i = 1, nMax do
				nLeft = (i-1)/nMax
				nRight = i/nMax
				self.tDividers[i] = Apollo.LoadForm(self.xmlDoc, "DividerTemp", self.wndResourceBar, self)
				self.tDividers[i]:SetAnchorPoints(nLeft, 0, nRight, 1)
				self.tDividers[i]:SetAnchorOffsets(0,0,0,0)
				--self.tDividers[i]:Show(true);
				--self.tDividers[i]:SetOpacity(0.5)
				--self.tDividers[i]:SetBGColor("vdarkgray")
				--self.tDividersTap[i]:SetSprite(self.Settings.playerBar.sTexture)
				local myPixie = {
					bLine = true,
					fWidth = 2.0,
					bRestart = true,
					cr = {a=1,r=0,g=0,b=0},
					loc = {
						fPoints = {nRight, 0, nRight, 1},
						nOffsets = {0,0,0,0}
					},
				}
				
				if i ~= nMax then					
					self.wndResourceBar:AddPixie(myPixie)
				end
			end
		elseif nMax == 2 and self.playerClass == GameLib.CodeEnumClass.Engineer then
			self.tDividers[1] = Apollo.LoadForm(self.xmlDoc, "DividerTemp", self.wndResourceBar, self)
			self.tDividers[1]:SetAnchorPoints(0, 0, 0.3, 1)
			self.tDividers[1]:SetAnchorOffsets(0,0,0,0)
			self.tDividers[2] = Apollo.LoadForm(self.xmlDoc, "DividerTemp", self.wndResourceBar, self)
			self.tDividers[2]:SetAnchorPoints(0.3, 0, 0.7, 1)
			self.tDividers[2]:SetAnchorOffsets(0,0,0,0)
			local myPixie = {
					bLine = true,
					fWidth = 2.0,
					bRestart = true,
					cr = {a=1,r=0,g=0,b=0},
					loc = {
						fPoints = {0.3, 0, 0.3	, 1},
						nOffsets = {0,0,0,0}
					},
				}
			self.wndResourceBar:AddPixie(myPixie)
			myPixie = {
					bLine = true,
					fWidth = 2.0,
					bRestart = true,
					cr = {a=1,r=0,g=0,b=0},
					loc = {
						fPoints = {0.7, 0, 0.7	, 1},
						nOffsets = {0,0,0,0}
					},
				}
			self.wndResourceBar:AddPixie(myPixie)
		end
		]]
		local CRBOptionsInterface = Apollo.GetAddon("OptionsInterface")
		if CRBOptionsInterface ~= nil and self.wndContainer ~= nil then
			local tWndSettings = CRBOptionsInterface.tTrackedWindows[self.wndContainer:GetId()] 
			if tWndSettings ~= nil then
				self.Settings.bLocked = not tWndSettings.bMoveable
			end
		end
		
		
		self.bIsLoaded = true
	end

end

function SlingDing:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("sd", "OnSlingDingOn", self)
		Apollo.RegisterEventHandler("VarChange_FrameCount", "OnFrame", self)
		

		
		
		self.wndConfig = Apollo.LoadForm(self.xmlDoc, "SlingDingConfig", nil, self)
		if self.wndConfig == nil then
			Apollo.AddAddonErrorText(self, "Could not load the Config for some reason.")
			return
		end
		self.wndConfig:Show(false, true)
		self.wndConfigMain = Apollo.LoadForm(self.xmlDoc, "Settings_Main", self.wndConfig:FindChild("SettingsFrame"), self)
		self.wndConfigMain:Show(true)

		if self.bWindowManagement then
			self:OnWindowManagementReady()
		end
		self:ApplySettings()
	end
	
end

function SlingDing:OnSave(eLevel)
    if eLevel == GameLib.CodeEnumAddonSaveLevel.Character then

	    return self.Settings
	end
end


function SlingDing:OnRestore(eLevel, tData)
	
    if eLevel == GameLib.CodeEnumAddonSaveLevel.Character then
		if (tData.version ~= defaultSettings.version and defaultSettings.bOverrideSaved) or tData.bOverrideSaved == nil then
			--self:ResetSettings()
			--Print("SlingDing - New Version of SlingDing. Incompatible saved data. SlingDing has been reset.")
			--self:MergeTables(tData , defaultSettings)
			local newData = {}
			for k,v in pairs(defaultSettings) do
				newData[k] = tData[k] or v
			end
			self.Settings = newData
		else 
			self.Settings = tData
		end
		self.bRestored = true
		
	
	end
end

function SlingDing:ResetSettings()
	self.Settings = self:CopyTable(defaultSettings)
	local playerUnit = GameLib.GetPlayerUnit()
	if playerUnit == nil then return end
	local playerClass = playerUnit:GetClassId()
	
	if self.bIsMelee or playerClass == GameLib.CodeEnumClass.Stalker or playerClass == GameLib.CodeEnumClass.Engineer or playerClass == GameLib.CodeEnumClass.Warrior then
		self.Settings.bFocus = false
		self.Settings.bFocusText = false
				
		self.Settings.bResourceText = true
		
		self.Settings.bResourceTextVCenter = true
		self.Settings.bResourceTextTop = false

	end
end

function SlingDing:OnWindowManagementReady()
	if self.wndContainer ~= nil then  
		Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndContainer, strName = "SlingDing"})
		
		-- Hooking into the OptionsInterface functions
		local CRBOptionsInterface = Apollo.GetAddon("OptionsInterface")
		if CRBOptionsInterface ~= nil then
			local CRBfunc = CRBOptionsInterface.OnMoveableChecked 
			CRBOptionsInterface.OnMoveableChecked =  function(self, wndHandler, wndControl)
				CRBfunc(CRBOptionsInterface, wndHandler, wndControl)
				local tSettings = wndControl:GetData()
				if tSettings.strName == "SlingDing" then
					local SD = Apollo.GetAddon("SlingDing")
					SD.OnWindowsManagementLocked(SD, not tSettings.bMoveable)
				end
				
			
			end
			local CRBfuncKey = CRBOptionsInterface.OnMoveableKeyChecked 
			CRBOptionsInterface.OnMoveableKeyChecked =  function(self, wndHandler, wndControl)
				CRBfuncKey(CRBOptionsInterface, wndHandler, wndControl)
				local tSettings = wndControl:GetData()
				if tSettings.strName == "SlingDing" then
					local SD = Apollo.GetAddon("SlingDing")
					SD.OnWindowsManagementLocked(SD, not tSettings.bMoveable)
				end
				
			
			end
		end
	end
	self.bWindowManagement = true
end

function SlingDing:OnWindowsManagementLocked(bLocked)
	self.Settings.bLocked = bLocked
	self:ApplySettings()
	
	if self.wndConfig:IsShown() then
		self:SetOptions()
		self.SettingsOld.bLocked = bLocked
	end

end


function SlingDing:OnFrame()
	
	
	if self.bIsLoaded then
		self:UpdateBars()
	end
	

end

function SlingDing:UpdateBars()
	if self.wndContainer:IsShown() == false then
		--Print("Invisible")
		return
	end

		
	local unitPlayer = GameLib.GetPlayerUnit()
	local nResourceCurr = unitPlayer:GetResource(self.Resource) or 0 
	local nResourceMax = unitPlayer:GetMaxResource(self.Resource) or 1
	local nFocusCurr = unitPlayer:GetMana() 
	local nFocusMax = unitPlayer:GetMaxMana() 
	local nFocusPercent = (nFocusCurr / nFocusMax)*100

	--if self.Settings.bEditMode then
	if self.wndConfig:IsShown() then
		nResourceCurr = nResourceMax * 0.8
		nFocusCurr = nFocusMax*0.8 
		nFocusPercent = (nFocusCurr / nFocusMax)*100	
	end
	
	--Focus
	self.wndFocusBar:SetMax(nFocusMax)
	self.wndFocusBar:SetProgress(nFocusCurr)
	local strFocusText = ""
	if not self.bIsMelee and  self.Settings.bFocusText then
		strFocusText = string.format ("%0s%%", nFocusPercent)
	end
	self.wndFocusText:SetText(strFocusText)
	
	local strResourceText = ""
	if self.Settings.bResourceText then
		strResourceText = string.format ("%0s", nResourceCurr)
	end
	self.wndResourceText:SetText(strResourceText)
	

	
	if self.playerClass == GameLib.CodeEnumClass.Medic then
		local playerBuffs = unitPlayer:GetBuffs()
		local powerCharge = 0
		local beneficialBuffs = playerBuffs["arBeneficial"]
		
		for key, value in pairs(beneficialBuffs) do
			if value.splEffect:GetId() == 42569 then 
				powerCharge = value.nCount
			end
		end
		nResourceCurr = nResourceCurr + (powerCharge/3)
	end	
	
	--Resource
	self.wndResourceBar:SetMax(nResourceMax)
	local nUpdateSpeed
	if self.Settings.bSmoothUpdate then
		nUpdateSpeed = nResourceMax/2
	end
	self.wndResourceBar:SetProgress(nResourceCurr, nUpdateSpeed)
	
	
	
	if self.tDividers ~= nil then
		local nResourcePercent = nResourceCurr / nResourceMax
		local tDiv = ktClassToDividers[self.playerClass]
		for k,wndDivider in pairs(self.tDividers) do
			wndDivider:Show(nResourcePercent >= tDiv[k]) --0.25

		end
	end	
	--[[	
	for i = 1, 4 do
		if i <= nDividers then
			--self.wndResourceBar:FindChild("Divider"..i):SetOpacity(0.8)
			self.wndResourceBar:FindChild("Divider"..i):Show(true)
		else
			--self.wndResourceBar:FindChild("Divider"..i):SetOpacity(0)
			self.wndResourceBar:FindChild("Divider"..i):Show(false)
		end	
	end]]
	
	-- Surge
	if self.playerClass == GameLib.CodeEnumClass.Spellslinger and  GameLib.IsSpellSurgeActive()== true  and self.Settings.bShowSurge then
		self.wndSurge:Show(true,true)
		
		--self.wndSurge:FindChild("BG"):SetOpacity(0.8)
		--self.wndSurge:FindChild("Flicker"):SetOpacity(0.15)
	else
		self.wndSurge:Show(false,false)
	end
	

end






function SlingDing:ApplySettings()
	if self.Settings == nil then
		--self.Settings = defaultSettings
		--self:CPrint("No Settings found")
		return
	end
	
	local tSettings = self.Settings

	local CRBOptionsInterface = Apollo.GetAddon("OptionsInterface")
	if CRBOptionsInterface ~= nil then
		local tWndSettings = CRBOptionsInterface.tTrackedWindows[self.wndContainer:GetId()] 
		if tWndSettings ~= nil and tWndSettings.bMoveable == self.Settings.bLocked then
			CRBOptionsInterface:OnMoveableChecked(tWndSettings.wndForm:FindChild("Moveable"), tWndSettings.wndForm:FindChild("Moveable"))
		end 
	end
	
	self.wndContainer:SetStyle("Sizable", not tSettings.bLocked)
	self.wndContainer:SetStyle("IgnoreMouse", tSettings.bLocked)
	
	-- Bar Height
	if tSettings.bFocus == true then
		self.wndResourceBar:SetAnchorPoints(0,0,1,1-tSettings.nResourceBarHeightPercent)
		self.wndResourceBar:SetAnchorOffsets(4,4,-4,-1)
		self.wndFocusBar:SetAnchorPoints(0,1-tSettings.nResourceBarHeightPercent,1,1)
		self.wndSurge:FindChild("Flicker"):SetAnchorPoints(0,0,1,1-tSettings.nResourceBarHeightPercent )
		self.wndSurge:FindChild("Flicker"):SetAnchorOffsets(4,4,-4,-1)
		self.wndFocusBar:Show(true)
		
	else
		self.wndResourceBar:SetAnchorPoints(0,0,1,1)
		self.wndResourceBar:SetAnchorOffsets(4,4,-4,-4)
		self.wndSurge:FindChild("Flicker"):SetAnchorPoints(0,0,1,1)
		self.wndSurge:FindChild("Flicker"):SetAnchorOffsets(4,4,-4,-4)
		self.wndFocusBar:Show(false)
	end

	
	
	--Textures
	self.wndResourceBar:SetFullSprite(tSettings.sResourceBarTexture)
	self.wndResourceBar:SetEmptySprite(tSettings.sResourceBarTexture)
	self.wndFocusBar:SetFullSprite(tSettings.sFocusBarTexture)
	self.wndFocusBar:SetEmptySprite(tSettings.sFocusBarTexture)
	for k,wndDivider in pairs(self.tDividers) do
		wndDivider:SetSprite(tSettings.sDividerTexture)
	end
	
	--Colors
	self.wndResourceBar:SetBarColor(tSettings.clrResourceBar)
	self.wndResourceBar:SetBGColor(tSettings.clrResourceBarBG)
	self.wndFocusBar:SetBarColor(tSettings.clrFocusBar)
	self.wndFocusBar:SetBGColor(tSettings.clrFocusBarBG)
	for k,wndDivider in pairs(self.tDividers) do
		wndDivider:SetBGColor(tSettings.clrDivider)
	end
	
	self.wndFocusText:SetTextColor(tSettings.clrFocusText)
	-- Opacity
	self.wndContainer:SetOpacity(tSettings.nOpacity)
	
	--Text
	self.wndFocusText:SetTextFlags("DT_CENTER", self.Settings.bFocusTextCenter)
	self.wndFocusText:SetTextFlags("DT_RIGHT", self.Settings.bFocusTextRight)
	self.wndFocusText:SetTextFlags("DT_VCENTER", self.Settings.bFocusTextVCenter)
	self.wndFocusText:SetTextFlags("DT_BOTTOM", self.Settings.bFocusTextBottom)
	
	self.wndResourceText:SetTextFlags("DT_CENTER", self.Settings.bResourceTextCenter)
	self.wndResourceText:SetTextFlags("DT_RIGHT", self.Settings.bResourceTextRight)
	self.wndResourceText:SetTextFlags("DT_VCENTER", self.Settings.bResourceTextVCenter)
	self.wndResourceText:SetTextFlags("DT_BOTTOM", self.Settings.bResourceTextBottom)
	
	--Surge
	self.wndSurge:FindChild("BG"):SetBGColor(tSettings.clrSurgeBG)
	self.wndSurge:FindChild("Flicker"):SetOpacity(tSettings.nFlickerOpacity)
	
end



function SlingDing:OnStanceBtn(wndHandler, wndControl)
	Pet_SetStance(0, tonumber(wndHandler:GetData())) -- First arg is for the pet ID, 0 means all engineer pets
	self.wndPetBar:FindChild("StanceMenuOpenerBtn"):SetCheck(false)
	self.wndPetBar:FindChild("PetText"):SetText(wndHandler:GetText())
	self.wndPetBar:FindChild("PetText"):SetData(wndHandler:GetText())
end

function SlingDing:OnPetBtn(wndHandler, wndControl)
	 local wndPetContainer = self.wndPetBar:FindChild("PetBarContainer")
	 wndPetContainer:Show(not wndPetContainer:IsShown())
end

function SlingDing:OnShowActionBarShortcut(nWhichBar, bIsVisible, nNumShortcuts)
	if nWhichBar ~= 1 or not self.wndPetBar or not self.wndPetBar:IsValid() then -- 1 is hardcoded to be the engineer pet bar
		return
	end

	self.wndPetBar:FindChild("PetBtn"):Show(bIsVisible)
	self.wndPetBar:FindChild("PetBarContainer"):Show(bIsVisible)
end

function SlingDing:OnEngineerPetBtnMouseEnter(wndHandler, wndControl)
	wndHandler:SetBGColor("white")
	local strHover = ""
	local strWindowName = wndHandler:GetName()
	if strWindowName == "ActionBarShortcut.12" then
		strHover = Apollo.GetString("ClassResources_Engineer_PetAttack")
	elseif strWindowName == "ActionBarShortcut.13" then
		strHover = Apollo.GetString("CRB_Stop")
	elseif strWindowName == "ActionBarShortcut.15" then
		strHover = Apollo.GetString("ClassResources_Engineer_GoTo")
	end
	self.wndPetBar:FindChild("PetText"):SetText(strHover)
end

function SlingDing:OnEngineerPetBtnMouseExit(wndHandler, wndControl)
	wndHandler:SetBGColor("UI_AlphaPercent50")
	self.wndPetBar:FindChild("PetText"):SetText(self.wndPetBar:FindChild("PetText"):GetData() or "")
end



function SlingDing:OnGeneratePetCommandTooltip(wndControl, wndHandler, eType, arg1, arg2)
	local xml = nil
	if eType == Tooltip.TooltipGenerateType_PetCommand then
		xml = XmlDoc.new()
		xml:AddLine(arg2)
		wndControl:SetTooltipDoc(xml)
	elseif eType == Tooltip.TooltipGenerateType_Spell then
		xml = XmlDoc.new()
		if arg1 ~= nil then
			xml:AddLine(arg1:GetFlavor())
		end
		wndControl:SetTooltipDoc(xml)
	end
end




-- on SlashCommand "/sd"
function SlingDing:OnSlingDingOn(sCmd, sInput )
	self:OnConfigure()
end





---------------------------------------------------------------------------------------------------
-- SlingDingConfig Functions
---------------------------------------------------------------------------------------------------
function SlingDing:SetOptions()

	local wnd = self.wndConfigMain
	--general
	wnd:FindChild("General"):FindChild("Opacity"):FindChild("TextBG"):FindChild("EditBox"):SetText(string.format("%.0f",self.Settings.nOpacity*100))
	wnd:FindChild("General"):FindChild("Window_EditMode:btnLock"):SetCheck(self.Settings.bLocked)
	
	--Spellsurged
	wnd:FindChild("Spellsurged"):FindChild("btnShowSurge"):SetCheck(self.Settings.bShowSurge)
	wnd:FindChild("Spellsurged"):FindChild("FlickerOpacity"):FindChild("TextBG"):FindChild("EditBox"):SetText(string.format("%.0f",self.Settings.nFlickerOpacity*100))
	wnd:FindChild("Spellsurged"):FindChild("SpellsurgeBGColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):SetBGColor(self.Settings.clrSurgeBG)
	
	-- Resource
	wnd:FindChild("ResourceBar"):FindChild("ResourceBarTexture"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):SetSprite(self.Settings.sResourceBarTexture)
	wnd:FindChild("ResourceBar"):FindChild("ResourceBarColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):SetBGColor(self.Settings.clrResourceBar)
	wnd:FindChild("ResourceBar"):FindChild("ResourceBarBGColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):SetBGColor(self.Settings.clrResourceBarBG)
	
	wnd:FindChild("ResourceBar"):FindChild("btnShowResourceText"):SetCheck(self.Settings.bResourceText)
	wnd:FindChild("ResourceBar"):FindChild("ResourceBarTextColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):SetBGColor(self.Settings.clrResourceText)
	
	wnd:FindChild("ResourceBar"):FindChild("TextHorizontalAlign"):FindChild("btnLeft"):SetCheck(self.Settings.bResourceTextLeft)
	wnd:FindChild("ResourceBar"):FindChild("TextHorizontalAlign"):FindChild("btnCenter"):SetCheck(self.Settings.bResourceTextCenter)
	wnd:FindChild("ResourceBar"):FindChild("TextHorizontalAlign"):FindChild("btnRight"):SetCheck(self.Settings.bResourceTextRight)
	
	wnd:FindChild("ResourceBar"):FindChild("TextVerticalAlign"):FindChild("btnTop"):SetCheck(self.Settings.bResourceTextTop)
	wnd:FindChild("ResourceBar"):FindChild("TextVerticalAlign"):FindChild("btnCenter"):SetCheck(self.Settings.bResourceTextVCenter)
	wnd:FindChild("ResourceBar"):FindChild("TextVerticalAlign"):FindChild("btnBottom"):SetCheck(self.Settings.bResourceTextBottom)
	
	wnd:FindChild("ResourceBar"):FindChild("btnSmoothUpdate"):SetCheck(self.Settings.bSmoothUpdate)
	
	-- focus bar
	wnd:FindChild("FocusBar"):FindChild("btnShowFocus"):SetCheck(self.Settings.bFocus)
	wnd:FindChild("FocusBar"):FindChild("FocusBarHeight"):FindChild("TextBG"):FindChild("EditBox"):SetText(string.format("%.0f",self.Settings.nResourceBarHeightPercent*100))
	wnd:FindChild("FocusBar"):FindChild("FocusBarTexture"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):SetSprite(self.Settings.sFocusBarTexture)
	wnd:FindChild("FocusBar"):FindChild("FocusBarColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):SetBGColor(self.Settings.clrFocusBar)
	wnd:FindChild("FocusBar"):FindChild("FocusBarBGColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):SetBGColor(self.Settings.clrFocusBarBG)
	
	wnd:FindChild("FocusBar"):FindChild("btnShowFocusText"):SetCheck(self.Settings.bFocusText)
	wnd:FindChild("FocusBar"):FindChild("FocusBarTextColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):SetBGColor(self.Settings.clrFocusText)
	
	wnd:FindChild("FocusBar"):FindChild("TextHorizontalAlign"):FindChild("btnLeft"):SetCheck(self.Settings.bFocusTextLeft)
	wnd:FindChild("FocusBar"):FindChild("TextHorizontalAlign"):FindChild("btnCenter"):SetCheck(self.Settings.bFocusTextCenter)
	wnd:FindChild("FocusBar"):FindChild("TextHorizontalAlign"):FindChild("btnRight"):SetCheck(self.Settings.bFocusTextRight)
	
	wnd:FindChild("FocusBar"):FindChild("TextVerticalAlign"):FindChild("btnTop"):SetCheck(self.Settings.bFocusTextTop)
	wnd:FindChild("FocusBar"):FindChild("TextVerticalAlign"):FindChild("btnCenter"):SetCheck(self.Settings.bFocusTextVCenter)
	wnd:FindChild("FocusBar"):FindChild("TextVerticalAlign"):FindChild("btnBottom"):SetCheck(self.Settings.bFocusTextBottom)
	
	--dividers
	wnd:FindChild("Divider"):FindChild("DividerTexture"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):SetSprite(self.Settings.sDividerTexture)
	wnd:FindChild("Divider"):FindChild("DividerColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):SetBGColor(self.Settings.clrDivider)
	
	
	if self.bIsMelee then
		if self.playerClass ~= GameLib.CodeEnumClass.Engineer then
			self.wndConfigMain:FindChild("Divider"):Show(false,true)
		else
			local l,t,r,b = self.wndConfigMain:FindChild("Divider"):GetAnchorOffsets()
			local nHeight = b-t
			l,t,r,b = self.wndConfigMain:FindChild("FocusBar"):GetAnchorOffsets()
			self.wndConfigMain:FindChild("Divider"):SetAnchorOffsets(l,t,r,t+nHeight)
		end
		self.wndConfigMain:FindChild("FocusBar"):Show(false,true)
		--self.wndConfigMain:SetAnchorOffsets(0,0,0,620)
	end
	if self.playerClass ~= GameLib.CodeEnumClass.Spellslinger then
		wnd:FindChild("Spellsurged"):Show(false,false)
	end

end

function SlingDing:GetOptions()
	
	local wnd = self.wndConfigMain

	-- General
	self.Settings.nOpacity = tonumber(wnd:FindChild("General"):FindChild("Opacity"):FindChild("TextBG"):FindChild("EditBox"):GetText())/100
	self.Settings.bLocked = wnd:FindChild("General"):FindChild("Window_EditMode:btnLock"):IsChecked()
	
	--  Spellsurge Effect
	self.Settings.bShowSurge = wnd:FindChild("Spellsurged"):FindChild("btnShowSurge"):IsChecked()
	self.Settings.clrSurgeBG = self:ConvertColorToHex(wnd:FindChild("Spellsurged"):FindChild("SpellsurgeBGColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):GetBGColor())
	self.Settings.nFlickerOpacity = tonumber(wnd:FindChild("Spellsurged"):FindChild("FlickerOpacity"):FindChild("TextBG"):FindChild("EditBox"):GetText())/100

	-- Resource Bar
	self.Settings.sResourceBarTexture = wnd:FindChild("ResourceBar"):FindChild("ResourceBarTexture"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):GetSprite()
	self.Settings.clrResourceBar = self:ConvertColorToHex(wnd:FindChild("ResourceBar"):FindChild("ResourceBarColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):GetBGColor())
	self.Settings.clrResourceBarBG = self:ConvertColorToHex(wnd:FindChild("ResourceBar"):FindChild("ResourceBarBGColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):GetBGColor())
	
	self.Settings.bResourceText = wnd:FindChild("ResourceBar"):FindChild("btnShowResourceText"):IsChecked()
	self.Settings.clrResourceText = self:ConvertColorToHex(wnd:FindChild("ResourceBar"):FindChild("ResourceBarTextColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):GetBGColor())
	
	self.Settings.bResourceTextLeft = wnd:FindChild("ResourceBar"):FindChild("TextHorizontalAlign"):FindChild("btnLeft"):IsChecked()
	self.Settings.bResourceTextCenter = wnd:FindChild("ResourceBar"):FindChild("TextHorizontalAlign"):FindChild("btnCenter"):IsChecked()
	self.Settings.bResourceTextRight = wnd:FindChild("ResourceBar"):FindChild("TextHorizontalAlign"):FindChild("btnRight"):IsChecked()
	
	self.Settings.bResourceTextTop = wnd:FindChild("ResourceBar"):FindChild("TextVerticalAlign"):FindChild("btnTop"):IsChecked()
	self.Settings.bResourceTextVCenter = wnd:FindChild("ResourceBar"):FindChild("TextVerticalAlign"):FindChild("btnCenter"):IsChecked()
	self.Settings.bResourceTextBottom = wnd:FindChild("ResourceBar"):FindChild("TextVerticalAlign"):FindChild("btnBottom"):IsChecked()
	self.Settings.bSmoothUpdate = wnd:FindChild("ResourceBar"):FindChild("btnSmoothUpdate"):IsChecked()

	--Focus bar
	self.Settings.bFocus = wnd:FindChild("FocusBar"):FindChild("btnShowFocus"):IsChecked()
	self.Settings.nResourceBarHeightPercent = tonumber(wnd:FindChild("FocusBar"):FindChild("FocusBarHeight"):FindChild("TextBG"):FindChild("EditBox"):GetText())/100
	self.Settings.sFocusBarTexture = wnd:FindChild("FocusBar"):FindChild("FocusBarTexture"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):GetSprite()
	self.Settings.clrFocusBar = self:ConvertColorToHex(wnd:FindChild("FocusBar"):FindChild("FocusBarColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):GetBGColor())
	self.Settings.clrFocusBarBG = self:ConvertColorToHex(wnd:FindChild("FocusBar"):FindChild("FocusBarBGColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):GetBGColor())

	self.Settings.bFocusText = wnd:FindChild("FocusBar"):FindChild("btnShowFocusText"):IsChecked()
	self.Settings.clrFocusText = self:ConvertColorToHex(wnd:FindChild("FocusBar"):FindChild("FocusBarTextColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):GetBGColor())
	
	self.Settings.bFocusTextLeft = wnd:FindChild("FocusBar"):FindChild("TextHorizontalAlign"):FindChild("btnLeft"):IsChecked()
	self.Settings.bFocusTextCenter = wnd:FindChild("FocusBar"):FindChild("TextHorizontalAlign"):FindChild("btnCenter"):IsChecked()
	self.Settings.bFocusTextRight = wnd:FindChild("FocusBar"):FindChild("TextHorizontalAlign"):FindChild("btnRight"):IsChecked()
	
	self.Settings.bFocusTextTop = wnd:FindChild("FocusBar"):FindChild("TextVerticalAlign"):FindChild("btnTop"):IsChecked()
	self.Settings.bFocusTextVCenter = wnd:FindChild("FocusBar"):FindChild("TextVerticalAlign"):FindChild("btnCenter"):IsChecked()
	self.Settings.bFocusTextBottom = wnd:FindChild("FocusBar"):FindChild("TextVerticalAlign"):FindChild("btnBottom"):IsChecked()
	
	--Divider
	self.Settings.sDividerTexture = wnd:FindChild("Divider"):FindChild("DividerTexture"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):GetSprite()
	self.Settings.clrDivider = self:ConvertColorToHex(wnd:FindChild("Divider"):FindChild("DividerColor"):FindChild("ColorSwatchBG"):FindChild("ColorSwatch"):GetBGColor())	

	

end

function SlingDing:OnOptionsOKClick(wndHandler, wndControl, eMouseButton)
	if wndHandler ~= wndControl then return end

	self:GetOptions()
	self.Settings.bEditMode = false
	self:ApplySettings()
	self.SettingsOld = nil
	
	self.wndConfig:Show(false)
end

function SlingDing:OnOptionsCancelClick(wndHandler, wndControl, eMouseButton)
	if wndHandler ~= wndControl then return end

	self.Settings = self:CopyTable(self.SettingsOld)
	self.SettingsOld = nil
	
	self:ApplySettings()

	self.wndConfig:Show(false)

end

function SlingDing:OnOptionsResetClick(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	
	self:ResetSettings()
	self.Settings.bEditMode = true
	
	-- Bar posi?elself.wndInterface:FindChild("Content:WindowContent:List")

	local CRBOptionsInterface = Apollo.GetAddon("OptionsInterface")
	if CRBOptionsInterface ~= nil then
		local tWndSettings = CRBOptionsInterface.tTrackedWindows[self.wndContainer:GetId()] 
		CRBOptionsInterface:OnResetBtn(tWndSettings.wndForm:FindChild("ResetBtn"),tWndSettings.wndForm:FindChild("ResetBtn"))
		--OptionsInterface:OnMoveableChecked(tWndSettings.wndForm:FindChild("ResetBtn"), tWndSettings.wndForm:FindChild("ResetBtn"))
		--tSettings.wndForm:FindChild("ResetBtn"):GetData()
		
		--OptionsInterface.tTrackedWindowsByName[tWndSettings.strName].tCurrentLoc = tWndSettings.tDefaultLoc
		--tWndSettings.wnd:MoveToLocation(WindowLocation.new(tWndSettings.tDefaultLoc))	
		--OptionsInterface:UpdateTrackedWindow(tWndSettings.wnd)
	end
	
	self:SetOptions()
	self:ApplySettings()

end

function SlingDing:OnOptionsAnyChange(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	self:GetOptions()
	self:ApplySettings()
end


function SlingDing:OnOptionsBarColorClick( wndHandler, wndControl )
	if wndControl  ~= wndHandler then return end
	if ColorPicker then
		local wndColor = wndHandler:FindChild("ColorSwatch")
		local clr = wndColor:GetBGColor()
		self.curCPColor = CColor.new( clr.r, clr.g, clr.b, clr.a)
		ColorPicker.AdjustCColor(self.curCPColor, true, SlingDingUpdateColor, wndColor)
	else 
		Print("Please install ColorPicker to change colors.")
	end 
end
function SlingDingUpdateColor(wndColor)
	local sd = Apollo.GetAddon("SlingDing")
	wndColor:SetBGColor(sd:ConvertColorToHex(sd.curCPColor))
	sd:GetOptions()
	sd:ApplySettings()
end


function SlingDing:OnOptionsTextureClick(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	local MS = MediaShare
	if MS then
		local curTexture = wndHandler:FindChild("ColorSwatch"):GetSprite()
		MS.GetTexture(curTexture, SlingDingUpdateTexture, wndHandler:FindChild("ColorSwatch"))
	end

end
function SlingDingUpdateTexture(newTexture, wnd)
	if wnd == nil or newTexture == nil then return end
	wnd:SetSprite(newTexture)
	Apollo.GetAddon("SlingDing"):OnOptionsAnyChange(true, true)
end





-----------------------------------------------------------------------------------------------
-- Helper Functions
-----------------------------------------------------------------------------------------------

function SlingDing:ConvertColorToHex(color)
	return string.format("%02x%02x%02x%02x", self:Round(255 * color.a), self:Round(255 * color.r), self:Round(255 * color.g), self:Round(255 * color.b))
end

function SlingDing:ConvertHexToColor(hex)
	local a = tonumber(string.sub(hex, 1, 2), 16)
	hex = string.sub(hex, -6)
	local r, g, b = 0, 0, 0
	local n = tonumber(hex, 16)
	if n then
		r = math.floor(n / 65536)
		g = math.floor((n / 256) % 256)
		b = n % 256
	end
	return r / 255, g / 255, b / 255, a / 255
end

function SlingDing:Round(n, nDecimals, bForceDecimals)
	local nMult = 1
	if nDecimals and nDecimals > 0 then
		for i = 1, nDecimals do nMult = nMult * 10 end
	end
	n = math.floor((n * nMult) + 0.5) / nMult
	if bForceDecimals then n = string.format("%." .. nDecimals .. "f", n) end
	return n
end


function SlingDing:CopyTable(t)
	local o = {}

	for k,v in pairs(t) do
		if type(v) == 'table' then
			o[k] = self:CopyTable(v)
		else
			o[k] = v
		end
	end

	return o
end




-----------------------------------------------------------------------------------------------
-- SlingDing Instance
-----------------------------------------------------------------------------------------------
local SlingDingInst = SlingDing:new()
SlingDingInst:Init()